;
;Simple NMI handler for OSI C2/C3/C4/C8
;Will print NMI address and registers to serial port upon interrupt
;Will need to be reinstalled if BASIC is loaded from disk as BASIC swaps stack & zero page out

;code lives in stack between $0130 and $01BF, just below IRQ handler
;To load, start monitor and load code.

;To generate NMI, pull pin 6 of 6502 low temporarily, perhaps with a 470ohm resistor to ground.
;Although multiple interrupts may be generated, they should mostly be ignored when one is being processed
;Mark Spankus 3/25/2024


ACIA=$FC00	;change to $F000 for C1P

*=$0130-7
	;0129 Set C3/65A Go address (reset vector) Y=6C, X=FC, A=FF, P=4, k=FD PCh PCl
	.BYTE $6C,$FC,$FF,$04,$FD,$01,$29
	
*=$0130	;NMI entry point
	;CLD
	PHA			;we push A
INIRQ=*+1
	LDA #$00
	BNE NOIRQ	;avoid recursive interrupts
	INC INIRQ
	TXA
	PHA			;we push X
	TYA
	PHA			;we push Y

LOOP
	JSR PRNMSG
OFFSET=*+1
	LDA $0108,X ;since X is set in subroutine (TSX), need to add 2 to get real values
	JSR PHEXA
	DEC OFFSET
	CPY #$00
	BNE LOOP

	JSR PRNMSG
	TXA

	ADC #$07	;carry is set from prnmsg
	JSR PHEXA	;print stack pointer before interrupt
	
	LDX #$06	;restore modified values
	STX PRNIDX
	LDX #$08
	STX OFFSET
	
IRQEXIT			;prepare to return to regularly scheduled pgm
	PLA
	TYA
	PLA
	TAX
	DEC INIRQ	;2 instruction window of nested interrupts
NOIRQ
	PLA
	RTI	

PRNMSG
	LDX #$00
PRNIDX=*+1
	LDY #$06	;start with last msg 1st
MLOOP
	LDA MSG,X
	BEQ CHKY
	CPY #$00
	BNE NXTONE	;not interested in this msg, don't print
	JSR OUTPUT
NXTONE
	INX
	BNE MLOOP
CHKY
	LDA #$3D
	INX
	DEY
	BPL MLOOP
	DEC PRNIDX
	LDY PRNIDX
	TSX
	BNE OUTPUT	;assume K is never 0


; PRINT HEX BYTE (A)
PHEXA
	PHA
	LSR	A
	LSR	A
	LSR	A
	LSR	A
	JSR	PHEXA1
	PLA
PHEXA1
	AND	#$F
	ORA	#'0'
	CMP	#'9'+1
	BMI	PHEXA2
	CLC
	ADC	#7
PHEXA2
	
OUTPUT
	PHA
OUTPUT1
	LDA ACIA		;wait for TxEmpty
	LSR A
	LSR A
	BCC OUTPUT1
	PLA
	STA ACIA+1
	RTS

MSG
	.BYTE " K",0
	.BYTE " Y",0
	.BYTE " X",0
	.BYTE " A",0
	.BYTE " P",0
	.BYTE 0
	.BYTE $0D,$0A,"NI",0
	

